@@ -1,12 +1,9 @@
 import json
-import logging
-import os
 import sys
-from pathlib import Path
 
 import yaml
 
-from .._implementation.configuration import Config
+from _appmap.configuration import Config
 
 
 def _run():
@@ -26,3 +23,7 @@ def _run():
 
 def run():
     sys.exit(_run())
+
+
+if __name__ == "__main__":
+    run()
